clear;clc;
path(path,'fastkmeans');
path(path,'anchors');
path(path,'BTSPLH');
path(path,'BTNSPLH');

%% Load Dataset
% fea_train all training data
% gnd_train all training data labels
% fea_vad labeled training data
% gnd_train labeled training data labels
% fea_test testing data
% gnd_test testing labels
dataname='mnist';
load(strcat('data/',dataname));

%% Computing Pairwise Label Matrix S %%
[gnd_vad sindex]=sort(gnd_vad);
fea_vad=fea_vad(sindex,:);
S = getS(gnd_vad);
 
%% Computing Anchors
fprintf('### Get Anchors ### \n');
r=randperm(size(fea_train,1));
maxiter = 10;
Nanchors = 300;
anchorfile=strcat('anchors/',dataname,'_anchors_',num2str(Nanchors),'.mat');
if ~exist(anchorfile)
    %%% compute
    use_samples = size(fea_train,1);
    start_t=clock;
    U = fastkmeans(fea_train(r(1:use_samples),:), Nanchors, 2, maxiter);
    end_t=clock;
    kmeansT=etime(end_t,start_t);
    save(anchorfile,'U');
else
    %%% load
    load(anchorfile);
end

%% Get Z for training data
fprintf('### Get Z ### \n');
theta=0.01;
s=2;
Zfile=strcat('anchors/',dataname,'_Z_',num2str(Nanchors),'_',num2str(s),'.mat');
if~(exist(Zfile))
    %%% compute
    [Z_train Z_vad meanZ_train getztime] = getZ(fea_train, fea_vad, U, s, theta);
    save(Zfile, 'Z_train', 'Z_vad', 'meanZ_train');
else
    %%% load
    load(Zfile)
end

%% Get Z for testing data
fprintf('### Get testZ ### \n');
testZfile=strcat('anchors/',dataname,'_testZ_',num2str(Nanchors),'_',num2str(s),'.mat');
if~(exist(testZfile))
    %%% compute
    [Z_test gettestztime] = getTestZ(fea_test, U, s, theta);
    save(testZfile, 'Z_test');
else
    %%% load
    load(testZfile);
end

%% max test bit length
maxbit=64;

%% Bootstrap Squential Projection Learing Hashing(BT-SPLH)
fprintf('### BTSPLH training ###\n');
BTSPLHparam.nbits=maxbit;
BTSPLHparam.lambda=0.04;
%these three parameters should be carefully tuned
BTSPLHparam.eta=16;
BTSPLHparam.alpha=0;
BTSPLHparam.beta=0;
[BTSPLHparam BTSPLHtrainT] = trainBTSPLH(fea_train, fea_vad, S, BTSPLHparam);
[BTSPLH_Y_train BTSPLH_compress_trainT] = compressBTSPLH(fea_train, BTSPLHparam);
[BTSPLH_Y_test BTSPLH_compress_testT] = compressBTSPLH(fea_test, BTSPLHparam);

%% Bootstrap Nonliear Squential Projection Learing Hashing(BT-NSPLH)
fprintf('### BTNSPLH training ###\n');
BTNSPLHparam.nbits=maxbit;
BTNSPLHparam.r=Nanchors;
BTNSPLHparam.lambda=1;
%these three parameters should be carefully tuned
BTNSPLHparam.eta=16;
BTNSPLHparam.alpha=-0.6;
BTNSPLHparam.beta=0;
[BTNSPLHparam BTNSPLHtrainT] = trainBTNSPLH(Z_train, Z_vad, meanZ_train, S, BTNSPLHparam);
[BTNSPLH_Y_train BTNSPLH_compress_trainT] = compressBTNSPLHtrain(Z_train, BTNSPLHparam);
[BTNSPLH_Y_test BTNSPLH_compress_testT] = compressBTNSPLH(Z_test, BTNSPLHparam);

%% Evaluation
start_t=clock;
nbits_set=[8 16 24 32 48 64];
fprintf('### Evaluation ### \n');
for i=1:length(nbits_set)
    nbits=nbits_set(i);
    fprintf('### The number of bits is %d ### \n',nbits);
     
   %% BTSPLH
    B_train = compactbit(BTSPLH_Y_train(:,1:nbits));
    B_test = compactbit(BTSPLH_Y_test(:,1:nbits));
    [BTSPLH_eva_info{i} BTSPLH_evaT] = evaluate(B_train, B_test, gnd_train, gnd_test);
    AP_BTSPLH(i)=BTSPLH_eva_info{i}.AP;
    PH2_BTSPLH(i)=BTSPLH_eva_info{i}.PH2;
    fprintf('BTSPLH %d bits -- AP: %f PH2: %f \n',nbits,AP_BTSPLH(i),PH2_BTSPLH(i));
    
   %% BTNSPLH
    B_train = compactbit(BTNSPLH_Y_train(:,1:nbits));
    B_test = compactbit(BTNSPLH_Y_test(:,1:nbits));
    [BTNSPLH_eva_info{i} BTNSPLH_evaT] = evaluate(B_train, B_test, gnd_train, gnd_test);
    AP_BTNSPLH(i)=BTNSPLH_eva_info{i}.AP;
    PH2_BTNSPLH(i)=BTNSPLH_eva_info{i}.PH2;
    fprintf('BTNSPLH %d bits -- AP: %f PH2: %f \n',nbits,AP_BTNSPLH(i),PH2_BTNSPLH(i)); 

end

%% Plot Results
%% MAP
figure;hold on;
plot(nbits_set,AP_BTSPLH,'-bh');
plot(nbits_set,AP_BTNSPLH,'-kp');
legend('BTSPLH','BTNSPLH');
ylabel('MAP');
xlabel('Number of bits');
grid on;
box on;

%% PH2
figure;hold on;
plot(nbits_set,PH2_BTSPLH,'-bh');
plot(nbits_set,PH2_BTNSPLH,'-kp');
legend('BTSPLH','BTNSPLH');
ylabel('PH2');
xlabel('Number of bits');
grid on;
box on;

end_t=clock;
evaT=etime(end_t,start_t);