function [Z_test time]=getTestZ(fea_test, U, s, theta)
%getTestZ Nonlinear Transformation for Testing Data
%
%	Usage:
%       [Z_test time]=getTestZ(fea_test, U, s, theta)
% 
%             Input:
%               fea_test   - Data matrix. Each row vector of fea is a data point.
%               U          - cluster centers.
%               s          - number of nearest neighbor.
%               theta      - bandwidth.
%
%             Output:
%               Z_test    - Nonlinear transformed data matrix. Each row vector of fea is a data point.
%               time       -  Time cost
%
%   Written by Chenxia Wu (chenxiawu AT hotmail.com)

start_t=clock;
[Ntest dim] = size(fea_test); 
[knum dim]=size(U);
Z_test=zeros(Ntest,knum);
for i=1:Ntest
%     disp(i);
    disl=U-repmat(fea_test(i,:),[knum 1]);
    disl=disl.^2;
    dis=sum(disl,2);
    [rv ri]=sort(dis);
    for j=1:s
        Z_test(i,ri(j))=exp(-sqrt(rv(j))*theta);
    end
end
Z_test=Z_test./repmat(sum(Z_test,2),[1,knum]);
end_t=clock;
time=etime(end_t,start_t);