function [Z_train Z_vad meanZ_train time]=getZ(fea_train, fea_vad, U, s, theta)
%getZ Nonlinear Transformation for Training Data
%
%	Usage:
%       [Z_train Z_vad meanZ_train time]=getZ(fea_train, fea_vad, U, s, theta)
% 
%             Input:
%               fea_train  - Data matrix. Each row vector is a data point.
%               fea_vad    - Labeled data matrix. Each row vector is a data point.
%               U          - cluster centers. Each row vector is a center.
%               s          - number of nearest neighbor.
%               theta      - bandwidth.
%
%             Output:
%               Z_train    - Nonlinear transformed data matrix. Each row vector is a data point.
%               Z_vad      - Nonlinear transformed labeled data matrix. Each row vector is a data point.
%               meanZ_train  - mean of each dimension.
%               time       -  Time cost
%               *Z_train and Z_vad are with zeromean
%
%   Written by Chenxia Wu (chenxiawu AT hotmail.com)

start_t=clock;
[Ntrain dim]=size(fea_train);
[Nvad dim]=size(fea_vad);
[Nu dim]=size(U);

Z_vad=zeros(Nvad,Nu);
Z_train=zeros(Ntrain,Nu);
for i=1:Nvad
    disl=U-repmat(fea_vad(i,:),[Nu 1]);
    disl=disl.^2;
    dis=sum(disl,2);
    [rv ri]=sort(dis);
    for j=1:s
        Z_vad(i,ri(j))=exp(-sqrt(rv(j))*theta);
    end
end
Z_vad=Z_vad./repmat(sum(Z_vad,2),[1,Nu]);

blocksize=1000;
blocNu=floor(Ntrain/blocksize);
for i=1:blocNu
%     disp(i);
    tbfea=fea_train((i-1)*blocksize+1:i*blocksize,:);
    for ii=1:blocksize
        disl=U-repmat(tbfea(ii,:),[Nu 1]);
        disl=disl.^2;
        dis=sum(disl,2);
        [rv ri]=sort(dis);
        for j=1:s
            Z_train((i-1)*blocksize+ii,ri(j))=exp(-sqrt(rv(j))*theta);
        end
    end
end
for i=blocksize*blocNu+1:Ntrain
    disl=U-repmat(fea_train(i,:),[Nu 1]);
        disl=disl.^2;
        dis=sum(disl,2);
        [rv ri]=sort(dis);
        for j=1:s
            Z_train(i,ri(j))=exp(-sqrt(rv(j))*theta);
        end
end
for i=1:Ntrain
    Z_train(i,:)=Z_train(i,:)/sum(Z_train(i,:));
end
meanZ_train=mean(Z_train);
Z_train=zeromean(Z_train);
% Z_train=Z_train-repmat(meanZ_train,[Ntrain 1]);
Z_vad=Z_vad-repmat(meanZ_train,[Nvad,1]);
end_t=clock;
time=etime(end_t,start_t);